==> pan-0.12.93.tar.bz2: pan-0.12.93/pan/base/util-file.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <glib.h>

#include <pan/base/base-prefs.h>
#include <pan/base/log.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/util-file.h>

/***
****
****  TEMP FILES
****
***/


char *
pan_make_temp (FILE ** setme_fp)
{
	GError * err = NULL;
	char * retval = NULL;
	int fd = g_file_open_tmp ("pan_XXXXXX", &retval, &err);

	if (err == NULL)
		*setme_fp = fdopen (fd, "w+");
	else {
		log_add_va (LOG_ERROR, _("Could not create temporary file: %s"), err->message);
		g_error_free (err);
	}

	return retval;
}


/***
****
****  FILES
****
***/


char*
pan_normalize_filename_inplace (char * filename)
{
	register char * in;
	register char * out;

	g_return_val_if_fail (is_nonempty_string(filename), filename);

	for (in=out=filename; *in; )
		if (in[0]==G_DIR_SEPARATOR && in[1]==G_DIR_SEPARATOR)
			++in;
		else
			*out++ = *in++;
	*out = '\0';

	return filename;
}


gboolean
file_exists (const char* filename)
{
	return is_nonempty_string(filename) && g_file_test (filename, G_FILE_TEST_EXISTS);
}

size_t
get_filesize (const char* filename)
{
	struct stat buf;

	g_return_val_if_fail (is_nonempty_string(filename), -1);

	if (!stat(filename, &buf))
		return buf.st_size;

	return -1;
}

gboolean
directory_check (const char *pathname)
{
	gboolean retval = TRUE;

	g_return_val_if_fail (is_nonempty_string(pathname), TRUE);

	if (!g_file_test (pathname, G_FILE_TEST_IS_DIR))
	{
		const gchar * in = pathname;
		gchar * buf = g_alloca (strlen(pathname)+1);
		gchar * out = buf;

		*out++ = *in++;
		for (;;)
		{
			if (*in=='\0' || *in==G_DIR_SEPARATOR)
			{
       				*out = '\0';

				if (!g_file_test (buf, G_FILE_TEST_IS_DIR))
				{
					if (mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR))
					{
						log_add_va (LOG_ERROR, _("Couldn't create directory \"%s\": %s"),
						                       buf, g_strerror (errno));
						retval = FALSE;
						break;
					}
				}
			}

			*out = *in;
			if (!*in)
				break;

			++out;
			++in;
		}
	}

	if (!retval)
		log_add_va (LOG_ERROR, _("Unable to access or create directory \"%s\""), pathname);

	return retval;
}

/***
****
****  MBOX
****
***/

const gchar*
mbox_get_next_message (const gchar   * text,
                       gchar        ** setme_from,
                       GString       * setme_msg)
{
	const gchar * march;
	gboolean have_from = FALSE;

	/* find the end of the message */
	g_string_truncate (setme_msg, 0);
	march = text;
	for (;;)
	{
		gboolean is_from;
		gint line_len = 0;
		const gchar * line_begin = NULL;
		const gchar * prev_march = march;

		if (!get_next_token_run (march, '\n', &march, &line_begin, &line_len))
			break;

		/* Is this a From line? */
		is_from = line_len>5
			&& line_begin[0]=='F'
			&& !strncmp (line_begin, "From ", 5);

		if (is_from && have_from) /* beginning of next message */
		{
			march = prev_march;
			break;
		}
		else if (is_from) /* beginning of our message */
		{
			have_from = TRUE;
			if (*setme_from != NULL)
				*setme_from = g_strndup (line_begin, line_len);
		}
		else if (have_from) /* a line in our message */
		{
			/* is this a nested From_ line? */
			if (*line_begin=='>') {
				const gchar * pch = line_begin;
				while (*pch=='>')
					++pch;
				if (*pch=='F' && !strncmp(pch,"From ", 5)) {
					gint diff = pch - line_begin;
					line_begin += diff;
					line_len -= diff;
				}
			}

			/* append the line */
			g_string_append_len (setme_msg, line_begin, line_len);
			g_string_append_c (setme_msg, '\n');
		}
	}

	return is_nonempty_string(march) ? march : NULL;
}

gchar*
mbox_format_message (const gchar * message,
                     const gchar * author_address,
                     time_t date)
{
	gchar * retval;
	GString * tmp;
	const gchar * march;

	/* default author address */
	if (!is_nonempty_string(author_address)) {
		pan_warn_if_reached ();
		author_address = _("unknown@spammesenseless.com");
	}

	/* sanity clause */
	g_return_val_if_fail (message!=NULL, NULL);
	g_return_val_if_fail (author_address!=NULL, NULL);

	/* add the From_ line.  note ctime() adds the '\n' */
       	tmp = g_string_new (NULL);
	g_string_sprintf (tmp, "From %s %s", author_address, ctime(&date));

	/* walk through the message, adding each line, prepending
	   From_ lines with '>' as specified by the mbox format */
	march = message;
	for (;;)
	{
		const gchar * line_begin;
		const gchar * line_end;
		gboolean is_from = FALSE;
		gboolean is_nested_from = FALSE;

		/* ran out of text */
		if (march == NULL)
			break;

		/* delimit the line */
		line_begin = march;
		line_end = strchr (march, '\n');
		if (line_end != NULL)
			march = line_end + 1;
		else { /* this new line is the last one */
			line_end = line_begin + strlen(line_begin);
			march = NULL;
		}

		/* check the line for delimiter */
		is_from = !strncmp (line_begin, "From ", 5);

		/* is this a nested From_ line? */
		if (*line_begin) {
			const gchar * pch = line_begin;
			while (*pch=='>')
				++pch;
			if (!strncmp(pch,"From ", 5))
				is_nested_from = TRUE;
		}

		/* append the line to the output */
		g_string_sprintfa (tmp, "%s%*.*s\n",
			(is_from||is_nested_from) ? ">" : "",
			line_end-line_begin,
			line_end-line_begin,
			line_begin);
	}

	/* mbox message ends with a blank line */
	if (tmp->len != 0)
		g_string_append_c (tmp, '\n');

	/* cleanup */
	retval = tmp->len ? tmp->str : NULL;
	g_string_free (tmp, retval==NULL);

	return retval;
}

static int
pan_copy (const char * old_filename,
          const char * new_filename)
{
	FILE * in;
	FILE * out;
	int retval;
	gchar buf [4096];
	size_t size_read;

	g_return_val_if_fail (is_nonempty_string(old_filename), -1);
	g_return_val_if_fail (is_nonempty_string(new_filename), -1);

	in = fopen (old_filename, "r");
	if (in == NULL) {
		log_add_va (LOG_ERROR, _("Can't open file \"%s\""), old_filename);
		return -1;
	}
	out = fopen (new_filename, "w+");
	if (out == NULL) {
		log_add_va (LOG_ERROR, _("Can't open file \"%s\""), new_filename);
		fclose (in);
		return -1;
	}

	while ((size_read = fread (buf, sizeof(char), sizeof(buf), in)))
		fwrite (buf, sizeof(char), size_read, out);

	retval = ferror(in) || ferror(out) ? 1 : 0;
	fclose (in);
	fclose (out);
	return retval;
}

gboolean
pan_rename (const gchar * old_filename,
            const gchar * new_filename)
{
	g_return_val_if_fail (is_nonempty_string(old_filename), FALSE);
	g_return_val_if_fail (is_nonempty_string(new_filename), FALSE);

	if (rename (old_filename, new_filename)) {
		if (!pan_copy (old_filename, new_filename))
			unlink (old_filename);
		else {
			log_add_va (LOG_ERROR,
				_("Error renaming \"%s\" as \"%s\": %s."),
				old_filename,
				new_filename,
				g_strerror(errno));
			return FALSE;
		}
	}

	return TRUE;
}

gint
change_file_mode_rw (FILE * fp, const gchar * file)
{
#if HAVE_FCHMOD
	return fchmod(fileno(fp), S_IRUSR|S_IWUSR);
#else
	return chmod(file, S_IRUSR|S_IWUSR);
#endif
}

==> pan-0.13.2.92.tar.bz2: pan-0.13.2.92/pan/base/util-file.c <==
/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <glib.h>

#include <pan/base/base-prefs.h>
#include <pan/base/log.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/util-file.h>

/***
****
****  TEMP FILES
****
***/


char *
pan_file_make_temp (FILE ** setme_fp)
{
	GError * err = NULL;
	char * retval = NULL;
	int fd = g_file_open_tmp ("pan_XXXXXX", &retval, &err);

	if (err == NULL)
		*setme_fp = fdopen (fd, "w+");
	else {
		log_add_va (LOG_ERROR, _("Could not create temporary file: %s"), err->message);
		g_error_free (err);
	}

	return retval;
}


/***
****
****  FILES
****
***/


char*
pan_file_normalize_inplace (char * filename)
{
	register char * in;
	register char * out;

	g_return_val_if_fail (is_nonempty_string(filename), filename);

	for (in=out=filename; *in; )
		if (in[0]==G_DIR_SEPARATOR && in[1]==G_DIR_SEPARATOR)
			++in;
		else
			*out++ = *in++;
	*out = '\0';

	return filename;
}


gboolean
pan_file_exists (const char* filename)
{
	return is_nonempty_string(filename) && g_file_test (filename, G_FILE_TEST_EXISTS);
}

size_t
pan_file_get_size (const char* filename)
{
	struct stat buf;

	g_return_val_if_fail (is_nonempty_string(filename), -1);

	if (!stat(filename, &buf))
		return buf.st_size;

	return -1;
}

gboolean
pan_file_ensure_path_exists (const char *pathname)
{
	gboolean retval = TRUE;
	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;

	g_return_val_if_fail (is_nonempty_string(pathname), TRUE);

	g_static_mutex_lock (&mutex);
	if (!g_file_test (pathname, G_FILE_TEST_IS_DIR))
	{
		const char * in = pathname;
		char * buf = g_alloca (strlen(pathname)+1);
		char * out = buf;

		*out++ = *in++;
		for (;;)
		{
			if (*in=='\0' || *in==G_DIR_SEPARATOR)
			{
       				*out = '\0';

				if (!g_file_test (buf, G_FILE_TEST_IS_DIR))
				{
					if (mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR))
					{
						log_add_va (LOG_ERROR, _("Couldn't create directory \"%s\": %s"),
						                       buf, g_strerror (errno));
						retval = FALSE;
						break;
					}
				}
			}

			*out = *in;
			if (!*in)
				break;

			++out;
			++in;
		}
	}
	g_static_mutex_unlock (&mutex);

	return retval;
}

static int
pan_file_copy (const char * old_filename,
               const char * new_filename)
{
	FILE * in;
	FILE * out;
	int retval;
	char buf [4096];
	size_t size_read;

	g_return_val_if_fail (is_nonempty_string(old_filename), -1);
	g_return_val_if_fail (is_nonempty_string(new_filename), -1);

	in = fopen (old_filename, "r");
	if (in == NULL) {
		log_add_va (LOG_ERROR, _("Error opening file \"%s\": %s"), old_filename, g_strerror(errno));
		return -1;
	}
	out = fopen (new_filename, "w+");
	if (out == NULL) {
		log_add_va (LOG_ERROR, _("Error opening file \"%s\": %s"), new_filename, g_strerror(errno));
		fclose (in);
		return -1;
	}

	while ((size_read = fread (buf, sizeof(char), sizeof(buf), in)))
		fwrite (buf, sizeof(char), size_read, out);

	retval = ferror(in) || ferror(out) ? 1 : 0;
	fclose (in);
	fclose (out);
	return retval;
}

gboolean
pan_file_rename (const char * old_filename,
                 const char * new_filename)
{
	g_return_val_if_fail (is_nonempty_string(old_filename), FALSE);
	g_return_val_if_fail (is_nonempty_string(new_filename), FALSE);

	if (rename (old_filename, new_filename)) {
		if (!pan_file_copy (old_filename, new_filename))
			unlink (old_filename);
		else {
			log_add_va (LOG_ERROR,
				_("Error renaming \"%s\" as \"%s\": %s."),
				old_filename,
				new_filename,
				g_strerror(errno));
			return FALSE;
		}
	}

	return TRUE;
}
